@@ -26,7 +26,8 @@ from proton.vpn.core.refresher import VPNDataRefresher
 
 from proton.vpn import logging
 from proton.vpn.connection import states, VPNConnection, events
-from proton.vpn.connection.exceptions import VPNConnectionError, AuthenticationError
+from proton.vpn.connection.exceptions import VPNConnectionError, \
+    AuthenticationError, HardJailedTwoFAError
 from proton.vpn.core.connection import VPNConnector
 
 from proton.vpn.app.gtk.services.reconnector.network_monitor import NetworkMonitor
@@ -148,6 +149,11 @@ class VPNReconnector:  # pylint: disable=too-many-instance-attributes
 
         raise VPNConnectionError(f"Reconnection not possible due to unexpected event: {event}")
 
+    def _on_two_fa_required(self):
+        raise HardJailedTwoFAError(
+            "Two factor authentication required to reconnect."
+        )
+
     def _on_session_unlocked(self):
         """
         Callback called by the session monitor once the user session has been
@@ -195,6 +201,10 @@ class VPNReconnector:  # pylint: disable=too-many-instance-attributes
         if isinstance(event, events.MaximumSessionsReached):
             return
 
+        if isinstance(event, events.TwoFARequired):
+            GLib.idle_add(self._on_two_fa_required)
+            return
+
         if not self.is_connection_error_fatal:  # noqa: E501 # pylint: disable=line-too-long # nosemgrep: python.lang.maintainability.is-function-without-parentheses.is-function-without-parentheses
             logger.info("VPN reconnection not possible: fatal connection error.")
             # Raise exception on the next event loop iteration so that the app reacts to it.
