@@ -20,11 +20,14 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with ProtonVPN.  If not, see <https://www.gnu.org/licenses/>.
 """
+from concurrent.futures import Future
 from typing import Union
 from gi.repository import GObject, Gtk
 
 from proton.vpn import logging
 from proton.vpn.app.gtk.controller import Controller
+from proton.vpn.app.gtk.util import connect_once
+from proton.vpn.app.gtk.utils.glib import add_done_callback
 from proton.vpn.app.gtk.widgets.main.notifications import Notifications
 from proton.vpn.app.gtk.widgets.main.loading_widget import OverlayWidget
 from proton.vpn.app.gtk.widgets.login.two_factor_auth.authenticator_app_form \
@@ -56,8 +59,22 @@ class TwoFactorAuthStack(Gtk.Stack):
         self._overlay_widget = overlay_widget
         self.active_widget = None
         self.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
-        self.security_key_form = None
 
+        # SecurityKeyForm
+        self.security_key_form = security_key_form or SecurityKeyForm(
+            controller, notifications, overlay_widget
+        )
+        self.security_key_form.connect(
+            "two-factor-auth-successful", self._on_two_factor_auth_successful
+        )
+        self.security_key_form.connect(
+            "two-factor-auth-cancelled", self._on_two_factor_auth_cancelled
+        )
+        self.add_titled(
+            self.security_key_form, "security_key_form", self.SECURITY_KEY_FORM_TITLE
+        )
+
+        # AuthenticatorAppForm
         self.authenticator_app_form = authenticator_app_form or AuthenticatorAppForm(
             controller, notifications, overlay_widget
         )
@@ -72,25 +89,6 @@ class TwoFactorAuthStack(Gtk.Stack):
             "two-factor-auth-cancelled", self._on_two_factor_auth_cancelled
         )
 
-        # Only show the security key form if environment variable is set and FIDO2 is available,
-        # otherwise it will be hidden by default (shows only authenticator app form).
-        # The environment variable should be removed once we fully deploy this feature,
-        # unless we implement a logic that hides it
-        # if a user doesn't have a security key configured.
-        if controller.fido2_available and controller.security_key_env_variable_set:
-            self.security_key_form = security_key_form or SecurityKeyForm(
-                controller, notifications, overlay_widget
-            )
-            self.security_key_form.connect(
-                "two-factor-auth-successful", self._on_two_factor_auth_successful
-            )
-            self.security_key_form.connect(
-                "two-factor-auth-cancelled", self._on_two_factor_auth_cancelled
-            )
-            self.add_titled(
-                self.security_key_form, "security_key_form", self.SECURITY_KEY_FORM_TITLE
-            )
-
     def display_widget(self, widget: Union[AuthenticatorAppForm, SecurityKeyForm]):
         """
         Displays the specified form to the user.
@@ -102,13 +100,32 @@ class TwoFactorAuthStack(Gtk.Stack):
     def reset(self):
         """Resets the widget to its initial state."""
         self._notifications.hide_message()
-        self.display_widget(self.authenticator_app_form)
+
+        if self._controller.fido2_available:
+            self.display_widget(self.security_key_form)
+            self.security_key_form.show()       # Enable Security Key
+            self.authenticator_app_form.show()  # Always enable Authenticator app
+        else:
+            self.security_key_form.hide()       # Disable Security Key
+            self.authenticator_app_form.show()  # Always enable Authenticator app
+            self.display_widget(self.authenticator_app_form)
 
     def _on_two_factor_auth_successful(self, _):
         self.emit("two-factor-auth-successful")
 
     def _on_two_factor_auth_cancelled(self, _):
-        self.emit("two-factor-auth-cancelled")
+        self._overlay_widget.show_message("Signing out...")
+        future = self._controller.logout()
+
+        def on_overlay_hidden(_overlay_widget: OverlayWidget, logout_future: Future):
+            logout_future.result()
+            self.emit("two-factor-auth-cancelled")
+
+        def on_logout(logout_future: Future):
+            connect_once(self._overlay_widget, "hide", on_overlay_hidden, logout_future)
+            self._overlay_widget.hide()
+
+        add_done_callback(future, on_logout)
 
     @GObject.Signal
     def two_factor_auth_successful(self):
